@@ -20,6 +20,8 @@
 
 from gi.repository import Gio, Gtk, GLib, Gdk
 
+import copy
+
 import appconsts
 from editorstate import APP
 from editorstate import current_sequence
@@ -27,6 +29,8 @@ from editorstate import PROJECT
 import editorpersistance
 import editorstate
 import gui
+import guiutils
+import respaths
 import snapping
 import translations
 import utils
@@ -86,14 +90,22 @@ _log_event_popover = None
 _log_event_menu = None
 _filter_mask_popover = None
 _filter_mask_menu = None
+_filter_mask_main_section = None
+_filter_mask_add_selected_sub_menu = None
+_filter_mask_add_all_sub_menu = None
 _filter_add_popover = None
 _filter_add_menu = None
 _render_args_popover = None
 _render_args_menu = None
 _kf_popover = None
 _kf_menu = None
-
-
+_edittools_popover = None
+_edittools_menu = None
+_rated_section = None
+_rated_submenu = None
+_ratings_radio_section = None
+_ratings_radio_submenu = None
+ 
 # -------------------------------------------------- menuitems builder fuctions
 def add_menu_action(menu, label, item_id, data, callback, active=True, app=None):
     if active == True:
@@ -109,6 +121,17 @@ def add_menu_action(menu, label, item_id, data, callback, active=True, app=None)
 
     return action
 
+def add_menu_action_icon(menu, label, icon, item_id, data, callback):
+    menu_item = Gio.MenuItem.new(label, "app." + item_id)
+    menu_item.set_icon(icon)
+    menu.append_item(menu_item)
+
+    action = Gio.SimpleAction(name=item_id)
+    action.connect("activate", callback, data)
+    APP().add_action(action)
+    
+    return action
+
 def add_menu_action_check(menu, label, item_id, checked_state, msg_str, callback):
     action = Gio.SimpleAction.new_stateful(name=item_id, parameter_type=None, state=GLib.Variant.new_boolean(checked_state))
     action.connect("activate", callback, msg_str)
@@ -150,8 +173,9 @@ def menu_clear_or_create(menu):
     
     return menu
 
-def new_popover(widget, menu, launcher):
+def new_popover(widget, menu, launcher, position_type=Gtk.PositionType.TOP):
     popover = Gtk.Popover.new_from_model(widget, menu)
+    popover.set_position(Gtk.PositionType(position_type))
     launcher.connect_launched_menu(popover)
     popover.show()
 
@@ -458,6 +482,7 @@ def media_hamburger_popover_show(launcher, widget, callback):
     _media_panel_hamburger_menu.append_section(None, proxy_section)
 
     order_section = Gio.Menu.new()
+    add_menu_action(order_section, _("Move Selected Items To Clicked Position..."), "mediapanel.movetoclicked", "move to clicked", callback)
     add_menu_action(order_section, _("Reverse Items Order"), "mediapanel.reverseorder", "reverse order", callback)
     _media_panel_hamburger_menu.append_section(None, order_section)
 
@@ -510,67 +535,102 @@ def columns_count_popupover_show(launcher, widget, callback):
     
 
 def file_filter_popover_show(launcher, widget, callback):
-    global _file_filter_popover, _file_filter_menu
+    global _file_filter_popover, _file_filter_menu, _ratings_radio_section, _ratings_radio_submenu
 
-    _file_filter_menu = menu_clear_or_create(_file_filter_menu)
-    items_data =[( _("All Files"), "0"), (_("Video Files"), "1"), \
-                ( _("Audio Files"), "2"), (_("Graphics Files"), "3"), ( _("Image Sequences"), "4"), \
-                (_("Pattern Producers"), "5"), (_("Unused Files"), "6")]
-                
-    active_index = editorstate.media_view_filter
-    radio_section = Gio.Menu.new()
-    add_menu_action_all_items_radio(radio_section, items_data, "mediapanel.fileview", active_index, callback)
-    _file_filter_menu.append_section(None, radio_section)
+    if _file_filter_menu == None:
+        
+        _file_filter_menu = menu_clear_or_create(_file_filter_menu)
+
+        items_data =[( _("All Files"), "0"), (_("Video Files"), "1"), \
+                    ( _("Audio Files"), "2"), (_("Graphics Files"), "3"), ( _("Image Sequences"), "4"), \
+                    (_("Pattern Producers"), "5"), (_("Unused Files"), "6")]
+                    
+        active_index = editorstate.media_view_filter
+        radio_section = Gio.Menu.new()
+        add_menu_action_all_items_radio(radio_section, items_data, "mediapanel.fileview", active_index, callback)
+        _file_filter_menu.append_section(None, radio_section)
+
+        ratings_items_data =[( _("Show All Ratings"), "show_all"), (_("Show Favorites"), "show_favorites"), \
+                    ( _("Hide Bad"), "hide_bad")]
+        ratings_active_index = editorstate.media_view_ratings_filter
+        _ratings_radio_section = Gio.Menu.new()
+        _ratings_radio_submenu = Gio.Menu.new()
+        add_menu_action_all_items_radio(_ratings_radio_submenu, ratings_items_data, "mediapanel.ratings", ratings_active_index, callback)
+        _ratings_radio_section.append_submenu(_("Ratings Filtering"), _ratings_radio_submenu)
+        _file_filter_menu.append_section(None, _ratings_radio_section)
+    else:
+        _ratings_radio_submenu = menu_clear_or_create(_ratings_radio_submenu)
+        ratings_items_data =[( _("Show All Ratings"), "show_all"), (_("Show Favorites"), "show_favorites"), \
+                    ( _("Hide Bad"), "hide_bad")]
+        ratings_active_index = editorstate.media_view_ratings_filter
+        add_menu_action_all_items_radio(_ratings_radio_submenu, ratings_items_data, "mediapanel.ratings", ratings_active_index, callback)
 
     _file_filter_popover = new_popover(widget, _file_filter_menu, launcher)
 
-def media_file_popover_show(media_file, widget, x, y, callback):
-    global _media_file_popover, _media_file_menu
+def media_file_popover_show(media_file, widget, x, y, callback, callback_rating):
+    global _media_file_popover, _media_file_menu, _rated_section, _rated_submenu
 
-    _media_file_menu = menu_clear_or_create(_media_file_menu)
+    if _media_file_menu == None:
+        _media_file_menu = menu_clear_or_create(_media_file_menu)
 
-    file_action_section = Gio.Menu.new()
-    add_menu_action(file_action_section, _("Rename"), "mediapanel.mediafile.rename", ("Rename", media_file), callback)
-    add_menu_action(file_action_section, _("Delete"), "mediapanel.mediafile.delete", ("Delete", media_file), callback)
-    _media_file_menu.append_section(None, file_action_section)
-    
-    if hasattr(media_file, "container_data"): 
-        if media_file.container_data == None:
-            monitor_item_active = True
+        file_action_section = Gio.Menu.new()
+        add_menu_action(file_action_section, _("Rename"), "mediapanel.mediafile.rename", ("Rename", media_file), callback)
+        add_menu_action(file_action_section, _("Delete"), "mediapanel.mediafile.delete", ("Delete", media_file), callback)
+        _media_file_menu.append_section(None, file_action_section)
+        
+        if hasattr(media_file, "container_data"): 
+            if media_file.container_data == None:
+                monitor_item_active = True
+            else:
+                monitor_item_active = False
         else:
-            monitor_item_active = False
+            monitor_item_active = True
+
+        if monitor_item_active == True:
+            monitor_section = Gio.Menu.new()
+            add_menu_action(monitor_section, _("Open in Clip Monitor"), "mediapanel.mediafile.clipmonitor", ("Open in Clip Monitor", media_file), callback)
+            _media_file_menu.append_section(None, monitor_section)
+
+        items_data =[(_("Unrated"), "unrated"), (_("Favorite"), "favorite"), \
+                    (_("Bad"), "bad")]
+        active_index = media_file.rating # Items indexes correspond with appconst values.
+
+        _rated_section = menu_clear_or_create(_rated_section)
+        _rated_submenu = menu_clear_or_create(_rated_submenu)
+        add_menu_action_all_items_radio(_rated_submenu, items_data, "mediapanel.mediafile", active_index, callback_rating)
+        _rated_section.append_submenu(_("Rating"), _rated_submenu)
+        _media_file_menu.append_section(None, _rated_section)
+        
+        if media_file.type != appconsts.PATTERN_PRODUCER:
+            properties_section = Gio.Menu.new()
+            add_menu_action(properties_section, _("File Properties"), "mediapanel.mediafile.fileproperties", ("File Properties", media_file), callback)
+            _media_file_menu.append_section(None, properties_section)
+
+        if hasattr(media_file, "container_data") == True and media_file.container_data == None:
+            if media_file.type != appconsts.PATTERN_PRODUCER and media_file.type != appconsts.AUDIO:
+                icon_section = Gio.Menu.new()
+                add_menu_action(icon_section, _("Replace Media In Project"), "mediapanel.mediafile.replace", ("Replace", media_file), callback)
+                add_menu_action(icon_section, _("Recreate Icon"), "mediapanel.mediafile.icon", ("Recreate Icon", media_file), callback)
+                _media_file_menu.append_section(None, icon_section)
+
+        if media_file.type != appconsts.IMAGE and media_file.type != appconsts.AUDIO and media_file.type != appconsts.PATTERN_PRODUCER:
+            render_section = Gio.Menu.new()
+            if media_file.type != appconsts.IMAGE_SEQUENCE:
+                add_menu_action(render_section, _("Render Slow/Fast Motion File"), "mediapanel.mediafile.slow", ("Render Slow/Fast Motion File", media_file), callback)
+            if media_file.type != appconsts.IMAGE_SEQUENCE:
+                add_menu_action(render_section, _("Render Reverse Motion File"), "mediapanel.mediafile.reverse", ("Render Reverse Motion File", media_file), callback)
+            _media_file_menu.append_section(None, render_section)
+                
+        if media_file.type == appconsts.VIDEO or media_file.type == appconsts.IMAGE_SEQUENCE:
+            proxy_section = Gio.Menu.new()
+            add_menu_action(proxy_section, _("Render Proxy File"), "mediapanel.mediafile.proxy", ("Render Proxy File", media_file), callback)
+            _media_file_menu.append_section(None, proxy_section)
     else:
-        monitor_item_active = True
-
-    if monitor_item_active == True:
-        monitor_section = Gio.Menu.new()
-        add_menu_action(monitor_section, _("Open in Clip Monitor"), "mediapanel.mediafile.clipmonitor", ("Open in Clip Monitor", media_file), callback)
-        _media_file_menu.append_section(None, monitor_section)
-
-    if media_file.type != appconsts.PATTERN_PRODUCER:
-        properties_section = Gio.Menu.new()
-        add_menu_action(properties_section, _("File Properties"), "mediapanel.mediafile.fileproperties", ("File Properties", media_file), callback)
-        _media_file_menu.append_section(None, properties_section)
-
-    if hasattr(media_file, "container_data") == True and media_file.container_data == None:
-        if media_file.type != appconsts.PATTERN_PRODUCER and media_file.type != appconsts.AUDIO:
-            icon_section = Gio.Menu.new()
-            add_menu_action(icon_section, _("Replace Media In Project"), "mediapanel.mediafile.replace", ("Replace", media_file), callback)
-            add_menu_action(icon_section, _("Recreate Icon"), "mediapanel.mediafile.icon", ("Recreate Icon", media_file), callback)
-            _media_file_menu.append_section(None, icon_section)
-
-    if media_file.type != appconsts.IMAGE and media_file.type != appconsts.AUDIO and media_file.type != appconsts.PATTERN_PRODUCER:
-        render_section = Gio.Menu.new()
-        if media_file.type != appconsts.IMAGE_SEQUENCE:
-            add_menu_action(render_section, _("Render Slow/Fast Motion File"), "mediapanel.mediafile.slow", ("Render Slow/Fast Motion File", media_file), callback)
-        if media_file.type != appconsts.IMAGE_SEQUENCE:
-            add_menu_action(render_section, _("Render Reverse Motion File"), "mediapanel.mediafile.reverse", ("Render Reverse Motion File", media_file), callback)
-        _media_file_menu.append_section(None, render_section)
-            
-    if media_file.type == appconsts.VIDEO or media_file.type == appconsts.IMAGE_SEQUENCE:
-        proxy_section = Gio.Menu.new()
-        add_menu_action(proxy_section, _("Render Proxy File"), "mediapanel.mediafile.proxy", ("Render Proxy File", media_file), callback)
-        _media_file_menu.append_section(None, proxy_section)
+        items_data =[(_("Unrated"), "unrated"), (_("Favorite"), "favorite"), \
+                    (_("Bad"), "bad")]
+        active_index = media_file.rating # Items indexes correspond with appconst values.
+        _rated_submenu = menu_clear_or_create(_rated_submenu)
+        add_menu_action_all_items_radio(_rated_submenu, items_data, "mediapanel.mediafile", active_index, callback_rating)
 
     rect = create_rect(x, y)
     _media_file_popover = new_mouse_popover(widget, _media_file_menu, rect)
@@ -857,22 +917,32 @@ def _add_filter_mask_submenu_items(sub_menu, filter_index, filter_names, filter_
         item_id = f_name.lower().replace(" ", "_") + str(data_bool)
         data = (data_bool, f_msg, filter_index)
         add_menu_action(sub_menu, label, item_id, data, callback)
-        
+
 def filter_mask_popover_show(launcher, widget, callback, filter_names, filter_msgs, filter_index):
-    global _filter_mask_popover, _filter_mask_menu
-    _filter_mask_menu = menu_clear_or_create(_filter_mask_menu)
+    global _filter_mask_popover, _filter_mask_menu, _filter_mask_main_section, \
+    _filter_mask_add_selected_sub_menu, _filter_mask_add_all_sub_menu
 
-    main_section = Gio.Menu.new()
-    add_selected_sub_menu = Gio.Menu.new()
-    main_section.append_submenu(_("Add Filter Mask on Selected Filter"), add_selected_sub_menu)
-    _add_filter_mask_submenu_items(add_selected_sub_menu, filter_index, filter_names, filter_msgs, False, callback)
+    if _filter_mask_menu == None:
+        _filter_mask_menu = menu_clear_or_create(_filter_mask_menu)
 
-    add_selected_sub_menu = Gio.Menu.new()
-    main_section.append_submenu(_("Add Filter Mask on All Filters"), add_selected_sub_menu)
-    _add_filter_mask_submenu_items(add_selected_sub_menu, filter_index, filter_names, filter_msgs, True, callback)
+        _filter_mask_main_section = Gio.Menu.new()
+        _filter_mask_add_selected_sub_menu = Gio.Menu.new()
+        _filter_mask_main_section.append_submenu(_("Add Filter Mask on Selected Filter"), _filter_mask_add_selected_sub_menu)
+        _add_filter_mask_submenu_items(_filter_mask_add_selected_sub_menu, filter_index, filter_names, filter_msgs, False, callback)
 
-    _filter_mask_menu.append_section(None, main_section)
-    _filter_mask_popover = new_popover(widget, _filter_mask_menu, launcher)
+        _filter_mask_add_all_sub_menu = Gio.Menu.new()
+        _filter_mask_main_section.append_submenu(_("Add Filter Mask on All Filters"), _filter_mask_add_all_sub_menu)
+        _add_filter_mask_submenu_items(_filter_mask_add_all_sub_menu, filter_index, filter_names, filter_msgs, True, callback)
+
+        _filter_mask_menu.append_section(None, _filter_mask_main_section)
+    else:
+        _filter_mask_add_selected_sub_menu = menu_clear_or_create(_filter_mask_add_selected_sub_menu)
+        _add_filter_mask_submenu_items(_filter_mask_add_selected_sub_menu, filter_index, filter_names, filter_msgs, False, callback)
+
+        _filter_mask_add_all_sub_menu = menu_clear_or_create(_filter_mask_add_all_sub_menu)
+        _add_filter_mask_submenu_items(_filter_mask_add_all_sub_menu, filter_index, filter_names, filter_msgs, True, callback)
+        
+    _filter_mask_popover = new_popover(widget, _filter_mask_menu, launcher, Gtk.PositionType.BOTTOM)
 
 def filter_add_popover_show(launcher, widget, clip, track, x, mltfiltersgroups, callback):
 
@@ -921,3 +991,88 @@ def render_args_popover_show(launcher, widget, callback):
     _render_args_menu.append_section(None, reset_section)
     
     _render_args_popover = new_popover(widget, _render_args_menu, launcher)
+
+def edittools_popover_show(launcher, toolsdata, widget, callback):
+    global _edittools_popover, _edittools_menu
+    _edittools_menu = menu_clear_or_create(_edittools_menu)
+    main_section = Gio.Menu.new()
+    for tool in toolsdata:
+        label, icon_name, item_id, data = tool
+        icon = Gio.FileIcon.new(Gio.File.new_for_path(respaths.IMAGE_PATH + icon_name))
+        add_menu_action_icon(main_section, label, icon, item_id, data, callback)
+
+    _edittools_menu.append_section(None, main_section)
+
+    _edittools_popover = new_popover(widget, _edittools_menu, launcher, Gtk.PositionType.BOTTOM)
+
+def edittools_popover_custom_show(launcher, toolsdata, widget, callback):
+    global _edittools_popover
+    
+    vbox = Gtk.VBox()
+    kb_shortcut = 1
+    tool_ids = []
+    for tool in toolsdata:
+        label_text, icon_name, item_id, data, tooltip = tool
+        tool_ids.append(data)
+        label = guiutils.get_left_justified_box([Gtk.Label.new(label_text)])
+        
+        tool_img = Gtk.Image.new_from_file(respaths.IMAGE_PATH + icon_name)
+        tool_img.set_size_request(30, 22)
+        
+        kb_shortcut_label = Gtk.Label.new(str(kb_shortcut))
+        kb_shortcut_label.set_size_request(22, 22)
+        guiutils.set_margins(kb_shortcut_label, 0,0,12,2)
+        
+        hbox = Gtk.HBox()
+        hbox.pack_start(tool_img, False, False, 0)
+        hbox.pack_start(label, True, True, 0)
+        hbox.pack_start(kb_shortcut_label, False, False, 0)
+        hbox.show_all()
+
+        menu_item = ToolMenuItem(data, hbox, tooltip, callback)
+
+        guiutils.set_margins(menu_item.widget, 4, 0, 4, 4)
+        vbox.pack_start(menu_item.widget, False, False, 0)
+        kb_shortcut += 1
+
+    vbox.show_all()
+    guiutils.set_margins(vbox, 0,4,0,0)
+        
+    _edittools_popover = Gtk.Popover.new(widget)
+    _edittools_popover.add(vbox)
+    _edittools_popover.set_position(Gtk.PositionType(Gtk.PositionType.BOTTOM))
+    _edittools_popover.connect("closed", lambda w: _shut_down_prelight(launcher))
+    _edittools_popover.show()
+
+def _shut_down_prelight(launcher):
+    launcher.shut_prelight()
+ 
+class ToolMenuItem:
+    
+    def __init__(self, tool_id, hbox, tooltip, callback):
+        color = gui.get_bg_color()
+        self.tool_id = tool_id
+        self.hbox = hbox
+        self.widget = Gtk.EventBox()
+        self.widget.connect("button-press-event", lambda w,e: callback(w, e, self.tool_id))
+        self.widget.connect('enter-notify-event', self._enter_notify_event)
+        self.widget.connect('leave-notify-event', self._leave_notify_event)
+        self.widget.add_events(Gdk.EventMask.KEY_PRESS_MASK)
+        self.widget.add_events(Gdk.EventMask.ENTER_NOTIFY_MASK)
+        self.widget.add_events(Gdk.EventMask.LEAVE_NOTIFY_MASK)
+        self.widget.set_tooltip_markup(tooltip)
+        self.widget.add(hbox)
+
+    def _enter_notify_event(self, widget, event):
+        color = Gdk.RGBA(red=1.0, green=1.0, blue=1.0, alpha=1.0)
+        color.parse("#393939")
+        self.widget.override_background_color(Gtk.StateType.NORMAL, color)
+        
+    def _leave_notify_event(self, widget, event):
+        color = Gdk.RGBA(red=1.0, green=1.0, blue=1.0, alpha=1.0)
+        color.parse("#292929")
+        self.widget.override_background_color(Gtk.StateType.NORMAL, color)
+
+def hide_edittools_popover():
+    global _edittools_popover
+    _edittools_popover.hide()
\ No newline at end of file
