@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2012 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ import mltfilters
 import mlttransitions
 import mltrefhold
 import patternproducer
-# import tlinerender
+import tlineypage
 import utils
 
 # Media types for tracks or clips
@@ -237,7 +237,7 @@ class Sequence:
         Creates a MLT playlist object, adds project
         data and adds to tracks list.
         """
-        new_track = mlt.Playlist()
+        new_track = mlt.Playlist(self.profile)
 
         self._add_track_attributes(new_track, track_type)
         new_track.is_sync_track = False
@@ -260,7 +260,7 @@ class Sequence:
         
         return new_track
 
-    def _add_track_attributes(self, track, type):                
+    def _add_track_attributes(self, track, type):
         # Add data attr
         track.type = type
         track.sequence = self
@@ -330,7 +330,7 @@ class Sequence:
             track = self.tracks[i]
             track.height = TRACK_HEIGHT_NORMAL
     
-        self.resize_tracks_to_fit(allocation)
+        tlineypage.vertical_size_update(allocation)
 
     def maximize_video_tracks_height(self, allocation):
         self.minimize_tracks_height()
@@ -338,7 +338,7 @@ class Sequence:
             track = self.tracks[i]
             track.height = TRACK_HEIGHT_NORMAL
     
-        self.resize_tracks_to_fit(allocation)
+        tlineypage.vertical_size_update(allocation)
 
     def maximize_audio_tracks_height(self, allocation):
         self.minimize_tracks_height()
@@ -346,7 +346,7 @@ class Sequence:
             track = self.tracks[i]
             track.height = TRACK_HEIGHT_NORMAL
     
-        self.resize_tracks_to_fit(allocation)
+        tlineypage.vertical_size_update(allocation)
 
     def get_tracks_height(self):
         h = 0
@@ -497,16 +497,17 @@ class Sequence:
         clip.sync_data = None 
         clip.mute_filter = None #
         clip.stream_indexes = None # a, v stream indexes when not muted
-        clip.clip_length = lambda: _clip_length(clip) # MLT get_length gives wrong values for blanks
+        clip.clip_length = lambda: _clip_length(clip) # MLT producer.get_length() gives producer max length.
         clip.waveform_data = None
         clip.color = None # None means that clip type default color is displayed
         clip.markers = []
         clip.container_data = None
-        
+        clip.titler_data = None # titler_data != None defines clips as a title clip that can be edited in Titler.
+
     def clone_track_clip(self, track, index):
         orig_clip = track.clips[index]
         if orig_clip.is_blanck_clip == True:
-            # Blank clips are created by adding a blank clip into MLT track and that is not approprite here,
+            # Blank clips are created by adding a blank clip into MLT track and that is not appropriate here,
             # so blank clips are represent by their length as int.
             return orig_clip.clip_length()
         return self.create_clone_clip(orig_clip)
@@ -533,7 +534,11 @@ class Sequence:
             clone_filter = mltfilters.clone_filter_object(f, self.profile)
             clone_clip.attach(clone_filter.mlt_filter)
             clone_clip.filters.append(clone_filter)
-    
+
+        # Image clips longer then 15000 frames need to have their length property set.
+        if clip.media_type == appconsts.IMAGE and clip.clip_out > 15000:
+            clone_clip.set("length", int(clip.clip_out - 1)) # -1, out is inclusive
+
     def copy_filters(self, clip, clone_clip):
         for f in clip.filters:
             clone_filter = mltfilters.clone_filter_object(f, self.profile)
@@ -620,11 +625,6 @@ class Sequence:
         return compositor
 
     def restack_compositors(self):
-        if self.compositing_mode == appconsts.COMPOSITING_MODE_STANDARD_FULL_TRACK:
-            # we should only see this on sequence creation and adding removing tracks for COMPOSITING_MODE_STANDARD_FULL_TRACK
-            # remove this later
-            print("restacking compositors!")
-
         self.sort_compositors()
 
         new_compositors = []
@@ -701,13 +701,13 @@ class Sequence:
         self.field.disconnect_service(old_compositor.transition.mlt_transition)
 
     def destroy_compositors(self):
-        # This can be called when undo stack destroyd too.
+        # This can be called when undo stack destroyed too.
         for compositor in self.compositors:
             self.field.disconnect_service(compositor.transition.mlt_transition)
         self.compositors = []
 
     def add_full_track_compositors(self):
-        print("Adding full track compositors")
+        #print("Adding full track compositors")
         
         for i in range(self.first_video_index, len(self.tracks) - 1):
             track = self.tracks[i]
@@ -724,7 +724,7 @@ class Sequence:
             self.add_compositor(compositor)
         
         self.restack_compositors()
-        print("Adding full track compositors DONE")
+        #print("Adding full track compositors DONE")
     
     def get_compositor_for_destroy_id(self, destroy_id):
         for comp in self.compositors:
@@ -806,7 +806,7 @@ class Sequence:
 
     def hide_hidden_clips(self):
         """
-        Called to temporarely remove hidden clips for trim mode loop playback
+        Called to temporarily remove hidden clips for trim mode loop playback
         """
         self.tracks[-1].clear()
         self._unmute_editable()
@@ -849,7 +849,7 @@ class Sequence:
         self._unmute_editable()
 
     def update_edit_tracks_length(self):
-        # Needed for timeline renderering updates
+        # Needed for timeline rendering updates
         self.seq_len = 0 
         for i in range(1, len(self.tracks) - 1):
             track_len = self.tracks[i].get_length()
@@ -864,11 +864,9 @@ class Sequence:
         seq_len = self.seq_len
         if seq_len < 1:
             seq_len = 1
-        
-        # tlinerender.get_renderer().update_hidden_track(self.tracks[-1], seq_len)
 
     def fix_v1_for_render(self): 
-        # This is a workaround to fix Issue #941 with H248 encoder not being able handle 
+        # This is a workaround to fix Issue #941 with H248 encoder not being able to handle 
         # blanks and crashing or losing working audio. Underlying reason still 
         # not known.
         track_v1 = self.tracks[self.first_video_index]
@@ -888,7 +886,7 @@ class Sequence:
             edit.append_clip(track_v1, white_clip, white_clip.clip_in, white_clip.clip_out)
 
     def get_seq_range_frame(self, frame):
-        # Needed for timeline renderering updates
+        # Needed for timeline rendering updates
         if frame >= (self.seq_len - 1):
             return self.seq_len - 1
         else:
@@ -985,7 +983,7 @@ class Sequence:
                 track_id += 1
                 continue
             elif track_id == len(self.tracks) - 2:
-                # This shold not happen because track heights should be set up so that minimized app 
+                # This should not happen because track heights should be set up so that minimized app 
                 fix_next = False
                 print("sequence.resize_tracks_to_fit (): could not make tracks fit in timeline vertical space")
             else:
