@@ -2,7 +2,7 @@
     Flowblade Movie Editor is a nonlinear video editor.
     Copyright 2022 Janne Liljeblad.
 
-    This file is part of Flowblade Movie Editor <http://code.google.com/p/flowblade>.
+    This file is part of Flowblade Movie Editor <https://github.com/jliljebl/flowblade/>.
 
     Flowblade Movie Editor is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@ import movemodes
 import panels
 import renderconsumer
 import render
+import respaths
 import userfolders
 
 # Used to store transition render data used at render complete callback.
@@ -148,7 +149,7 @@ def _add_transition_dialog_callback(dialog, response_id, selection_widgets, tran
     to_clip = transition_data["to_clip"]
 
     # Get values to build transition render sequence
-    # Divide transition lenght between clips, odd frame goes to from_clip 
+    # Divide transition length between clips, odd frame goes to from_clip 
     real_length = length + 1 # first frame is 100% a from_clip frame so we are going to have to drop that
     to_part = real_length // 2
     from_part = real_length - to_part
@@ -291,7 +292,7 @@ def re_render_transition(data):
         return
     
     from_clip_id, to_clip_id, from_out, from_in, to_out, to_in, transition_type_selection_index, \
-    sorted_wipe_luma_index, color_str = clip.creation_data
+    sorted_wipe_luma_index = clip.creation_data
     
     from_clip = editorstate.current_sequence().get_clip_for_id(from_clip_id)
     to_clip = editorstate.current_sequence().get_clip_for_id(to_clip_id)
@@ -327,7 +328,7 @@ def _transition_RE_render_dialog_callback(dialog, response_id, selection_widgets
     clip = transition_data["clip"]
     track =  transition_data["track"]
     from_clip_id, to_clip_id, from_out, from_in, to_out, to_in, transition_type_selection_index, \
-    sorted_wipe_luma_index, color_str = clip.creation_data
+    sorted_wipe_luma_index = clip.creation_data
     
     trans_index = track.clips.index(clip)
 
@@ -339,8 +340,7 @@ def _transition_RE_render_dialog_callback(dialog, response_id, selection_widgets
                                                                         to_out,
                                                                         to_in,
                                                                         transition_type_selection_index,
-                                                                        sorted_wipe_luma_index,
-                                                                        color_str)
+                                                                        sorted_wipe_luma_index)
     
 
     # Save transition data into global variable to be available at render complete callback
@@ -396,7 +396,7 @@ def _show_no_handles_dialog(from_req, from_handle, to_req, to_handle, length):
     img2 = Gtk.Image.new_from_file ((respaths.IMAGE_PATH + "transition_right.png"))
     img2.set_margin_bottom(24)
 
-    label1 = Gtk.Label(label=_("Current situation, not enought media overlap:"))
+    label1 = Gtk.Label(label=_("Current situation, not enough media overlap:"))
     label1.set_margin_bottom(12)
     label2 = Gtk.Label(label=_("You need more media overlap:"))
     label2.set_margin_bottom(12)
@@ -415,7 +415,6 @@ def _show_no_handles_dialog(from_req, from_handle, to_req, to_handle, length):
 
     rows = [row1, row2, row3, row4]
 
-
     if first_clip_info != None:
         row6 = guiutils.get_left_justified_box([label4])
         rows.append(row6)
@@ -604,7 +603,7 @@ class ReRenderderAllWindow:
         profile = PROJECT().profile
         folder = userfolders.get_render_dir()
         file_name = hashlib.md5(str(os.urandom(32)).encode('utf-8')).hexdigest()
-        self.write_file = folder + "/"+ file_name + file_ext
+        self.write_file = folder + file_name + file_ext
         consumer = renderconsumer.get_render_consumer_for_encoding_and_quality(self.write_file, profile, encoding_option_index, quality_option_index)
         
         if clip.rendered_type > appconsts.RENDERED_COLOR_DIP:
@@ -751,4 +750,10 @@ def _no_creation_data_dialog():
 def _source_clips_not_found_dialog():
     primary_txt = _("Can't rerender this fade / transition.")
     secondary_txt = _("The clip/s used to create this fade / transition are no longer available on the timeline.")
-    dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
\ No newline at end of file
+    dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
+    
+def _no_audio_tracks_mixing_info():
+    primary_txt = _("Rendered transitions cannot be used on audio tracks.")
+    secondary_txt = _("This feature only works on video tracks.")
+    dialogutils.info_message(primary_txt, secondary_txt, gui.editor_window.window)
+        
